package org.greenstone.gsdl3.gs3build.extractor;

import java.util.Iterator;

import org.greenstone.gsdl3.gs3build.doctypes.DocumentID;
import org.greenstone.gsdl3.gs3build.doctypes.DocumentInterface;
import org.greenstone.gsdl3.gs3build.doctypes.DocumentList;

public class ExtractorManager
{
  public static final String ACCUMULATE_MODE = "accumulate";

  DocumentList          documents;
  ExtractorInterface [] list;
  int                   size;
  int                   used;

  public ExtractorManager(DocumentList documentList)
  { this.list = new ExtractorInterface[10];
    this.size = 10;
    this.used = 0;
    this.documents = documentList;
  }

  public void addExtractor(ExtractorInterface extractor)
  { this.ensureSize(this.used + 1);
    this.list[this.used] = extractor;
    this.used ++;

    extractor.configure(this.documents);
  }

  public void extractDocument(DocumentID docId, DocumentInterface document)
  { for (int i = 0; i < this.used; i ++)
    { this.list[i].extractDocument(docId, document);
    }
  }

  public void extractDocuments()
  { for (int i = 0; i < this.used; i ++)
    { for (int p = 0; p < this.list[i].getNumberOfPasses(); p ++)
      {	this.list[i].startPass(p);

        Iterator iterator = documents.iterator();
	while (iterator.hasNext()) {
	  DocumentInterface document = (DocumentInterface) iterator.next();
	  this.list[i].extractDocument(document.getID(), document);
	}
	this.list[i].endPass(p);
      }
    }
  }

  public void ensureSize(int size)
  { while (size >= this.size)
    { ExtractorInterface newList [] = new ExtractorInterface[this.size*2];
      this.size *= 2;
      System.arraycopy(this.list, 0, newList, 0, this.size);
      this.list = newList;
    }
  }
}
